/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPICARD_H
#define WPICARD_H

#include "dart.h"
#include "wSolver.h"

#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Sparse>

namespace dart
{

/**
 * @brief Picard (fixed-point) solver for the Full Potential Equation
 * @authors Adrien Crovato
 */
class DART_API Picard : public Solver
{
public:
    double relax; ///< relaxation

public:
    Picard(std::shared_ptr<Problem> _pbl, std::shared_ptr<tbox::LinearSolver> _linsol, double rTol = 1e-6, double aTol = 1e-8, int mIt = 25, int nthrds = 1, int vrb = 1);

    virtual bool run() override;

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void build(Eigen::SparseMatrix<double, Eigen::RowMajor> &A, std::vector<double> &b);
};

} // namespace dart

#endif //WPICARD_H
