


def Dictionary(blScalars, blVectors, xtr):
  """Create dictionary with the coordinates and the boundary layer parameters
  """

  wData = {}
  # [xx, x, deltaStar, theta, H, N, Ue, Hk, Hstar, Hstar2, Cf, Cd, Ct, delta, VtExt, Me, Rhoe]
  wData['xx']         = blVectors[0]
  wData['x/c']        = blVectors[1]
  wData['deltaStar']  = blVectors[2]
  wData['theta']      = blVectors[3]
  wData['H']          = blVectors[4]
  wData['N']          = blVectors[5]
  wData['Ue']         = blVectors[6]
  wData['Ct']         = blVectors[7]
  wData['Hk']         = blVectors[8]
  wData['Hstar']      = blVectors[9]
  wData['HStar2']     = blVectors[10]
  wData['Cf']         = blVectors[11]
  wData['Cd']         = blVectors[12]
  wData['delta']      = blVectors[13]
  wData['VtExt']      = blVectors[14]
  wData['Me']         = blVectors[15]
  wData['Rhoe']       = blVectors[16]
  wData['xtrTop']     = xtr[0]
  wData['xtrBot']     = xtr[1]
  return wData

def WriteFile(wData, Re, toW=['delta*', 'H', 'Cf', 'Ctau']):
  """Write the results in airfoil_viscous.dat
  """
  
  # Write
  print('Writing file: airfoil_viscous.dat...', end = '')
  f = open('airfoil_viscous.dat', 'w+')

  f.write('$Aerodynamic coefficients\n')
  f.write('             Re              Cd             Cdp             Cdf         xtr_top         xtr_bot\n')
  f.write('{0:15.6f} {1:15.6f} {2:15.6f} {3:15.6f} {4:15.6f} {5:15.6f}\n'.format(Re, wData['Cd'], wData['Cdp'],
                                                                                  wData['Cdf'], wData['xtrTop'],
                                                                                  wData['xtrBot']))
  f.write('$Boundary layer variables\n')
  f.write('              x               y               z             x/c')

  for s in toW:
    f.write(' {0:>15s}'.format(s))
  f.write('\n')

  for i in range(len(self.data[:,0])):
    f.write('{0:15.6f} {1:15.6f} {2:15.6f} {3:15.6f}'.format(wData['x'][i], wData['y'][i], wData['z'][i], wData['x/c'][i]))
    for s in toW:
      f.write(' {0:15.6f}'.format(wData[s][i]))
    f.write('\n')

  f.close()
  print('done.')

def ExtractVars(viscSolver):
  import numpy as np

  blScalars = [viscSolver.Cdt, viscSolver.Cdf, viscSolver.Cdp]
  
  # [xx, x, deltaStar, theta, H, N, Ue, Hk, Hstar, Hstar2, Cf, Cd, Ct, delta, VtExt, Me, Rhoe]

  blVectors = viscSolver.ExtractSolution()


  return blScalars, blVectors

def PlotVars(wData, var='H'):
  from matplotlib import pyplot as plt

  nUpper = len(wData['x/c'])
  for i in range(len(wData['x/c'])):
    if wData['x/c'][i] == 1.0:
      nUpper = i+1
      break

  # [0   1          2      3  4  5   6   7      8       9  10  11  12     13     14  15    16]  
  # [xx, x, deltaStar, theta, H, N, Ue, Hk, Hstar, Hstar2, Cf, Cd, Ct, delta, VtExt, Me, Rhoe]
  
  plt.plot(wData['x/c'][:nUpper], wData[var][:nUpper], color='#0072BD', linewidth=3)
  plt.plot(wData['x/c'][nUpper:], wData[var][nUpper:], color='#D95319', linewidth=3)
  plt.axvline(x=wData['xtrTop'], linestyle="--", color='#0072BD')
  plt.axvline(x=wData['xtrBot'], linestyle="--", color='#D95319')
  plt.xlabel('$x/c$')
  plt.ylabel(var)
  plt.xlim([0,1])
  plt.legend(['Upper side', 'Lower side'], frameon=False)
  plt.show()